@@ -27,17 +27,16 @@ impl<V: IdentityVault> TrustPublicKeyPolicy<V> {
 
 #[async_trait]
 impl<V: IdentityVault> TrustPolicy for TrustPublicKeyPolicy<V> {
-    async fn check(&mut self, trust_info: &SecureChannelTrustInfo) -> Result<bool> {
-        let contact;
-        if let Some(c) = self
+    async fn check(&self, trust_info: &SecureChannelTrustInfo) -> Result<bool> {
+        let contact = if let Some(c) = self
             .identity
             .get_contact(trust_info.their_identity_id())
             .await?
         {
-            contact = c;
+            c
         } else {
             return Ok(false);
-        }
+        };
 
         if let Ok(pub_key) = contact.get_public_key(&self.public_key_label) {
             Ok(pub_key == self.public_key)
